
# q-expansion of newform 4732.2.g.d, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4732
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2367, [1, 0]], [2705, [1, 0]], [4565, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, -3], [0, 1], [0, -2], [0, 0], [7, 0], [0, -2], [4, 0], [1, 0], [0, -4], [0, 1], [0, 3], [6, 0], [0, -10], [-7, 0], [0, 6], [7, 0], [0, -8], [0, 6], [0, -11], [-14, 0], [0, 14], [0, -10], [0, -2], [-3, 0], [6, 0], [-14, 0], [0, -18], [-5, 0], [-18, 0], [-10, 0], [0, -9], [10, 0], [0, -13], [0, 18], [7, 0], [0, -14], [0, -22], [-18, 0], [-16, 0], [-19, 0], [8, 0], [0, -5], [0, 18], [26, 0], [-8, 0], [0, 18], [0, 12], [0, 10], [-6, 0], [0, 6], [0, -15], [10, 0], [-1, 0], [30, 0], [10, 0], [0, 2], [-29, 0], [0, -1], [-6, 0], [0, 27], [0, -4], [-20, 0], [-30, 0], [0, -29], [0, 18], [33, 0], [-32, 0], [0, 2], [0, -21], [0, -16], [-28, 0], [13, 0], [0, 16], [0, -6], [-17, 0], [0, -18], [0, -25], [0, -21], [-12, 0], [0, -25], [0, -24], [19, 0], [26, 0], [12, 0], [0, 6], [0, 1], [0, 33], [0, -30], [-6, 0], [0, -4], [0, 32], [32, 0], [0, -40], [-10, 0], [0, -15], [1, 0], [20, 0], [0, 9], [40, 0], [0, -3], [-14, 0], [42, 0], [20, 0], [0, 7], [0, -12], [0, 17], [18, 0], [41, 0], [10, 0], [0, -25], [0, -15], [0, 6], [0, 16], [-15, 0], [0, 12], [-6, 0], [6, 0], [20, 0], [0, 7], [33, 0], [18, 0], [0, 30], [0, 36], [18, 0], [0, -19], [42, 0], [-20, 0], [0, -11], [0, -4], [0, 0], [-10, 0], [6, 0], [0, -6], [0, -50], [0, 14], [0, 44], [-42, 0], [15, 0], [0, 56], [0, 42], [0, 0], [0, -6], [-51, 0], [0, 50], [0, 23], [7, 0], [-16, 0], [0, 54], [0, -13], [3, 0], [48, 0], [48, 0], [-14, 0], [4, 0], [-2, 0], [0, -1], [-11, 0], [0, 6], [0, -4], [-6, 0], [0, -32], [18, 0], [0, -7], [0, 10], [-28, 0], [15, 0], [0, 54], [33, 0], [0, 44], [0, 27], [-18, 0], [0, 21], [-28, 0], [-47, 0], [0, 38], [0, -63], [-32, 0], [-54, 0], [0, -32], [-56, 0], [29, 0], [0, -18], [0, -12], [-59, 0], [-14, 0], [0, -46], [0, 43], [0, 12], [41, 0], [0, 48], [8, 0], [0, 2], [66, 0], [38, 0], [0, 37], [11, 0], [0, 9], [42, 0], [0, 9], [-18, 0], [0, -23], [41, 0], [0, -66], [31, 0], [0, 38], [-6, 0], [0, 9], [40, 0], [45, 0], [-14, 0], [-10, 0], [0, 38], [0, 14], [0, -7], [68, 0], [-43, 0], [0, 28], [0, -62], [33, 0], [0, -58], [0, -59], [0, 32], [-12, 0], [-5, 0], [-15, 0], [48, 0], [38, 0], [0, -44], [-57, 0], [20, 0], [0, -52], [17, 0], [24, 0], [0, -14], [0, -34], [0, -31], [-54, 0], [30, 0], [0, 52], [-4, 0], [-52, 0], [0, -49], [0, -45], [-10, 0], [0, 60], [0, 12], [0, 22], [78, 0], [0, -18], [0, 38], [0, 48], [23, 0], [14, 0], [0, 24], [-10, 0], [0, -8], [-42, 0], [0, 39], [-32, 0], [32, 0], [0, 1], [39, 0], [0, 13], [12, 0], [0, -50], [0, -53], [0, -42], [15, 0], [29, 0], [0, 32], [0, -71], [-10, 0], [62, 0], [0, 60], [0, 10], [0, 13], [0, 7], [-58, 0], [-40, 0], [0, -74], [-30, 0], [0, 9], [0, 12], [78, 0], [15, 0], [0, 11], [0, 40], [3, 0], [2, 0], [-40, 0], [0, 57], [0, -48], [-26, 0], [-3, 0], [-2, 0], [-83, 0], [-40, 0], [0, -68], [59, 0], [0, -27], [-60, 0], [12, 0], [-6, 0], [0, 17], [-36, 0], [7, 0], [0, -82], [23, 0], [-52, 0], [0, -38], [-25, 0], [-2, 0], [0, -30], [61, 0], [0, -40], [0, 60], [0, 10], [-39, 0], [68, 0], [0, 26], [-65, 0], [0, -24], [0, -26], [65, 0], [0, -54], [0, 26], [-18, 0], [9, 0], [0, -31], [-47, 0], [2, 0], [0, 26], [0, 16], [0, 62], [0, 1], [0, 42], [0, 83], [22, 0], [0, -22], [87, 0], [0, -33], [-26, 0], [0, 18], [-30, 0], [67, 0], [0, -84], [0, 12], [22, 0], [0, -8], [0, 90], [0, 55], [4, 0], [7, 0], [69, 0], [0, 0], [0, 32], [-35, 0], [0, 6], [0, -57], [-45, 0], [2, 0], [0, -10], [-20, 0], [22, 0], [0, -14], [0, -26], [29, 0], [54, 0], [92, 0], [0, 10], [-39, 0], [32, 0], [1, 0], [0, 22], [54, 0], [0, 82], [-11, 0], [23, 0], [0, -22], [0, 35], [0, -42], [0, -30], [0, 58], [0, -54], [0, -82], [46, 0], [0, -52], [-28, 0], [0, -1], [0, 57], [0, -6], [70, 0], [0, -30], [22, 0], [0, -2], [-86, 0], [50, 0], [0, -91], [0, 2], [66, 0], [0, 88], [0, 57], [0, -25], [8, 0], [0, -22], [0, 49], [0, -42], [0, 72], [-29, 0], [-67, 0], [-48, 0], [-30, 0], [-11, 0], [-5, 0], [0, -90], [4, 0], [0, -6], [-74, 0], [-21, 0], [0, -3], [0, -104], [-12, 0], [0, 86], [0, -47], [-30, 0], [0, 65], [0, 22], [48, 0]]
